package com.gydx.component;

import java.io.IOException;
import java.sql.Timestamp;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.gydx.controller.BiddingMesController;
import com.gydx.service.BiddingMesService;
import com.gydx.service.RuleManagementService;
import com.gydx.vo.BiddingMesVO;
import com.gydx.vo.FoodRuleVO;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * ----------------------- 竞价详情 -----------------------
 * 1.点击开始竞价，建立链接
 * 2.依据点击时间判断现在的倒计时，持续发送倒计时
 * 3.到达休息时间倒计时时，传递休息信息，关闭报价按钮，显示第一轮报价
 * 4.休息倒计时结束，重新开始倒计时，打开关闭按钮，知道轮次结束
 * 5.传递关闭信息，更改对应单号的状态为已经结素，同时关闭这个socket链接
 */
@ServerEndpoint(value = "/client/BiddingDetail/{BiddingMesId}")
@Component
public class WSBiddingDetail {

    private static final Logger logger = LogManager.getLogger(WSBiddingDetail.class.getName());

    /**
     * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
     */
    private static CopyOnWriteArraySet<WSBiddingDetail> wsClientMap = new CopyOnWriteArraySet<>();

    /**
     * websocket 由于是多对象无法通过属性自动注入，解决方案：
     * 1.声明static类型的（需要注入的）属性---这里使用的就是这个
     * 2.通过set方法进行注入
     */
    private static BiddingMesService biddingMesService;

    @Autowired
    public void setSocketMessageHandler(BiddingMesService messageHandler) {
        biddingMesService = messageHandler;
    }

    private static RuleManagementService ruleManagementService;

    @Autowired
    public void setSocketMessageRuleID(RuleManagementService messageHandler) {
        ruleManagementService = messageHandler;
    }

    /**
     * 与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    private Session session;

    /**
     * 连接建立成功调用的方法
     *
     * @param session 当前会话session
     */
    @OnOpen
    public void onOpen(@PathParam("BiddingMesId") String BiddingMesId, Session session) {
        //建立链接
        this.session = session;
        //添加客户端对象
        wsClientMap.add(this);
        //日志记录
        logger.info(session.getId() + "有新链接加入，当前链接数为：" + wsClientMap.size());
        //倒计时逻辑
        SendCountdown(BiddingMesId);
    }

    /**
     * 连接关闭
     */
    @OnClose
    public void onClose() {
        wsClientMap.remove(this);
        logger.info("有一链接关闭，当前链接数为：" + wsClientMap.size());
    }

    /**
     * 收到客户端消息
     *
     * @param message 客户端发送过来的消息
     * @param session 当前会话session
     */
    @OnMessage
    public void onMessage(String message, Session session) throws IOException, InterruptedException {
        logger.info("来终端的警情消息:" + message);
    }

    /**
     * 发生错误
     */
    @OnError
    public void onError(Session session, Throwable error) {
        logger.info("wsClientMap发生错误!");
        error.printStackTrace();
    }

    /**
     * 给所有客户端群发消息
     *
     * @param message 消息内容
     */
    public void sendMsgToAll(String message) throws IOException {
        for (WSBiddingDetail item : wsClientMap) {
            item.session.getBasicRemote().sendText(message);
        }
        logger.info("成功群送一条消息:" + wsClientMap.size());
    }

    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
        logger.info("成功发送一条消息:" + message);
    }

    /**
     * 倒计时计算
     *
     * @param BiddingMesId 对应订单ID
     */
    public void SendCountdown(String BiddingMesId) {
        BiddingMesVO BMvo = new BiddingMesVO();
        BMvo.setBiddingMesID(BiddingMesId);
        List BMlist = biddingMesService.findBiddingMesByID(BMvo);

        if (BMlist.isEmpty()) {
            System.err.println("BMlist is empty");
            return;
        }

        BiddingMesVO voB = (BiddingMesVO) BMlist.get(0);
        Timestamp clickStartTime = voB.getClickStartTime();

        FoodRuleVO foodRuleVO = new FoodRuleVO();
        foodRuleVO.setFoodRuleID(voB.getFoodRuleID());
        List RFlist = ruleManagementService.queryRuleByID(foodRuleVO);
        FoodRuleVO voRF = (FoodRuleVO) RFlist.get(0);

        if (clickStartTime == null) {
            System.err.println("ClickStartTime is null");
            return;
        }

        // 计算每轮竞价时间和休息时间的总时长（以毫秒为单位）
        //竞价时间
        long roundsTimeMillis = voRF.getRoundsTime() * 60 * 1000;
        //休息时间
        long roundSpacingTimeMillis = voRF.getRoundSpacingTime() * 60 * 1000;
        long preparationTimeMillis = 60 * 1000; // 准备时间为1分钟

        // 计算总时长（不减去最后一次休息时间）
        long totalRoundsMillis = voRF.getTotalRounds() * (roundsTimeMillis + roundSpacingTimeMillis);

        // 计算结束时间（总时长减去最后一次休息时间，加上准备时间）
        Timestamp EndTimeDownCount = new Timestamp(clickStartTime.getTime() + totalRoundsMillis - roundSpacingTimeMillis + preparationTimeMillis);

        // Convert Timestamp to LocalDateTime
        ZonedDateTime targetTime = EndTimeDownCount.toInstant().atZone(ZoneId.systemDefault());

        // 每秒更新一次倒计时
        ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
        scheduler.scheduleAtFixedRate(() -> {
            //传递的数据
            Map<String, Object> data = new HashMap<>();
            long currentTimeMillis = System.currentTimeMillis();  // 只调用一次
            long timeLeftMillis = EndTimeDownCount.getTime() - currentTimeMillis;

            if (timeLeftMillis <= 0) {
                String status = "已结束";
                data.put("status", status);
                data.put("currentRound", "/");
                data.put("timeLeft", status);
                sendMessageToClient(data);
                scheduler.shutdown();
            } else if (timeLeftMillis > (totalRoundsMillis - roundSpacingTimeMillis)) {
                data.put("status", "准备中");
                data.put("currentRound", 0);
                String status = "即将开始";
                data.put("timeLeft", status);
                sendMessageToClient(data);
            } else {
                //一轮竞价总时长
                long roundDurationMillis = roundsTimeMillis + roundSpacingTimeMillis;
                //当前轮次剩余时间
                long currentRoundTimeLeft = timeLeftMillis % roundDurationMillis;
                // 已经过的总时间（单位：毫秒）
                long timeElapsedMillis = totalRoundsMillis - timeLeftMillis;
                //计算现在的轮数
                int completedRounds = 0;
                if (timeElapsedMillis > 0) {
                    completedRounds = (int) (timeElapsedMillis / roundDurationMillis);
                }
                // 当前轮数（从第一轮开始）
                int nowRounds = completedRounds + 1;
                data.put("currentRound", nowRounds);

                if (currentRoundTimeLeft > roundsTimeMillis) {
                    //休息中
                    data.put("status", "休息");
                } else {
                    //进行中
                    data.put("status", "进行中");
                }

                //实际发送的倒计时
                long minutes = (timeLeftMillis / 1000) / 60;
                long seconds = (timeLeftMillis / 1000) % 60;
                String timeLeft = String.format("%02d:%02d", minutes, seconds);
                data.put("timeLeft", timeLeft);
                sendMessageToClient(data);
            }
        }, 0, 1, TimeUnit.SECONDS);
    }


    private void sendMessageToClient(Object data) {
        try {
            if (session != null && session.isOpen()) {
                ObjectMapper objectMapper = new ObjectMapper();
                String jsonMessage = objectMapper.writeValueAsString(data);
                session.getBasicRemote().sendText(jsonMessage);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
